#!/usr/bin/env python

__version__ = '$Revision: 4791 $'.split()[1]
__date__ = '$Date: 2007-01-23 $'.split()[1]
__author__ = 'xmlbinmsg'

__doc__='''

Autogenerated python functions to serialize/deserialize binary messages.

Generated by: ./aisxmlbinmsg2py.py

Need to then wrap these functions with the outer AIS packet and then
convert the whole binary blob to a NMEA string.  Those functions are
not currently provided in this file.

serialize: python to ais binary
deserialize: ais binary to python

The generated code uses translators.py, binary.py, and aisstring.py
which should be packaged with the resulting files.


@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{BitVector<http://cheeseshop.python.org/pypi/BitVector>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ parser
@status: under development
@license: Generated code has no license
'''

import sys
from decimal import Decimal
from BitVector import BitVector

import binary, aisstring

# FIX: check to see if these will be needed
TrueBV  = BitVector(bitstring="1")
"Why always rebuild the True bit?  This should speed things up a bunch"
FalseBV = BitVector(bitstring="0")
"Why always rebuild the False bit?  This should speed things up a bunch"


fieldList = [
	'vessel',
	'direction',
	'ETA_month',
	'ETA_day',
	'ETA_hour',
	'ETA_min',
	'reserved',
]

def encode(params, validate=False):
	'''Create a sls_lockorder binary message payload to pack into an AIS Msg sls_lockorder.

	Fields in params:
	  - vessel(aisstr6): Vessel Name
	  - direction(bool): Up bound/Down bound
	  - ETA_month(uint): Estimated time of arrival  month 1..12
	  - ETA_day(uint): Estimated time of arrival  day of the month 1..31
	  - ETA_hour(uint): Estimated time of arrival  UTC hours 0..23
	  - ETA_min(uint): Estimated time of arrival  minutes
	  - reserved(uint): Reserved bits for future use (field automatically set to "0")
	@param params: Dictionary of field names/values.  Throws a ValueError exception if required is missing
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: BitVector
	@return: encoded binary message (for binary messages, this needs to be wrapped in a msg 8
	@note: The returned bits may not be 6 bit aligned.  It is up to you to pad out the bits.
	'''

	bvList = []
	if 'vessel' in params:
		bvList.append(aisstring.encode(params['vessel'],90))
	else:
		bvList.append(aisstring.encode('@@@@@@@@@@@@@@@',90))
	if params["direction"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['ETA_month']),4))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['ETA_day']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['ETA_hour']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['ETA_min']),6))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=0),19))

	return binary.joinBV(bvList)

def decode(bv, validate=False):
	'''Unpack a sls_lockorder message 

	Fields in params:
	  - vessel(aisstr6): Vessel Name
	  - direction(bool): Up bound/Down bound
	  - ETA_month(uint): Estimated time of arrival  month 1..12
	  - ETA_day(uint): Estimated time of arrival  day of the month 1..31
	  - ETA_hour(uint): Estimated time of arrival  UTC hours 0..23
	  - ETA_min(uint): Estimated time of arrival  minutes
	  - reserved(uint): Reserved bits for future use (field automatically set to "0")
	@type bv: BitVector
	@param bv: Bits defining a message
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: dict
	@return: params
	'''

	#Would be nice to check the bit count here..
	#if validate:
	#	assert (len(bv)==FIX: SOME NUMBER)
	r = {}
	r['vessel']=aisstring.decode(bv[0:90])
	r['direction']=bool(int(bv[90:91]))
	r['ETA_month']=int(bv[91:95])
	r['ETA_day']=int(bv[95:100])
	r['ETA_hour']=int(bv[100:105])
	r['ETA_min']=int(bv[105:111])
	r['reserved']=0
	return r

def decodevessel(bv, validate=False):
	return aisstring.decode(bv[0:90])

def decodedirection(bv, validate=False):
	return bool(int(bv[90:91]))

def decodeETA_month(bv, validate=False):
	return int(bv[91:95])

def decodeETA_day(bv, validate=False):
	return int(bv[95:100])

def decodeETA_hour(bv, validate=False):
	return int(bv[100:105])

def decodeETA_min(bv, validate=False):
	return int(bv[105:111])

def decodereserved(bv, validate=False):
	return 0


def printHtml(params, out=sys.stdout):
		out.write("<h3>sls_lockorder<h3>\n")
		out.write("<table border=\"1\">\n")
		out.write("<tr bgcolor=\"orange\">\n")
		out.write("<th align=\"left\">Field Name</th>\n")
		out.write("<th align=\"left\">Type</th>\n")
		out.write("<th align=\"left\">Value</th>\n")
		out.write("<th align=\"left\">Value in Lookup Table</th>\n")
		out.write("<th align=\"left\">Units</th>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>vessel</td>\n")
		out.write("<td>aisstr6</td>\n")
		if 'vessel' in params:
			out.write("	<td>"+str(params['vessel'])+"</td>\n")
			out.write("	<td>"+str(params['vessel'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>direction</td>\n")
		out.write("<td>bool</td>\n")
		if 'direction' in params:
			out.write("	<td>"+str(params['direction'])+"</td>\n")
			if str(params['direction']) in directionDecodeLut:
				out.write("<td>"+directionDecodeLut[str(params['direction'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>ETA_month</td>\n")
		out.write("<td>uint</td>\n")
		if 'ETA_month' in params:
			out.write("	<td>"+str(params['ETA_month'])+"</td>\n")
			out.write("	<td>"+str(params['ETA_month'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>ETA_day</td>\n")
		out.write("<td>uint</td>\n")
		if 'ETA_day' in params:
			out.write("	<td>"+str(params['ETA_day'])+"</td>\n")
			out.write("	<td>"+str(params['ETA_day'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>ETA_hour</td>\n")
		out.write("<td>uint</td>\n")
		if 'ETA_hour' in params:
			out.write("	<td>"+str(params['ETA_hour'])+"</td>\n")
			out.write("	<td>"+str(params['ETA_hour'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>ETA_min</td>\n")
		out.write("<td>uint</td>\n")
		if 'ETA_min' in params:
			out.write("	<td>"+str(params['ETA_min'])+"</td>\n")
			out.write("	<td>"+str(params['ETA_min'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>reserved</td>\n")
		out.write("<td>uint</td>\n")
		if 'reserved' in params:
			out.write("	<td>"+str(params['reserved'])+"</td>\n")
			out.write("	<td>"+str(params['reserved'])+"</td>\n")
		out.write("</tr>\n")
		out.write("</table>\n")

def printFields(params, out=sys.stdout, format='std', fieldList=None):
	'''Print a reserved message to stdout.

	Fields in params:
	  - vessel(aisstr6): Vessel Name
	  - direction(bool): Up bound/Down bound
	  - ETA_month(uint): Estimated time of arrival  month 1..12
	  - ETA_day(uint): Estimated time of arrival  day of the month 1..31
	  - ETA_hour(uint): Estimated time of arrival  UTC hours 0..23
	  - ETA_min(uint): Estimated time of arrival  minutes
	  - reserved(uint): Reserved bits for future use (field automatically set to "0")
	@param params: Dictionary of field names/values.  
	@param out: File like object to write to
	@rtype: stdout
	@return: text to out
	'''

	if 'std'==format:
		out.write("reserved:\n")
		if 'vessel' in params: out.write("	vessel:     "+str(params['vessel'])+"\n")
		if 'direction' in params: out.write("	direction:  "+str(params['direction'])+"\n")
		if 'ETA_month' in params: out.write("	ETA_month:  "+str(params['ETA_month'])+"\n")
		if 'ETA_day' in params: out.write("	ETA_day:    "+str(params['ETA_day'])+"\n")
		if 'ETA_hour' in params: out.write("	ETA_hour:   "+str(params['ETA_hour'])+"\n")
		if 'ETA_min' in params: out.write("	ETA_min:    "+str(params['ETA_min'])+"\n")
		if 'reserved' in params: out.write("	reserved:   "+str(params['reserved'])+"\n")
	elif 'csv'==format:
		if None == options.fieldList:
			options.fieldList = fieldList
		needComma = False;
		for field in fieldList:
			if needComma: out.write(',')
			needComma = True
			if field in params:
				out.write(str(params[field]))
			# else: leave it empty
		out.write("\n")
	elif 'html'==format:
		printHtml(params,out)
	elif 'sql'==format:
		sqlInsertStr(params,out)
	else: 
		print "ERROR: unknown format:",format
		assert False

	return # Nothing to return

directionEncodeLut = {
	'Down bound':'0',
	'Up bound':'1',
	} #directionEncodeLut

directionDecodeLut = {
	'0':'Down bound',
	'1':'Up bound',
	} # directionEncodeLut

######################################################################
# SQL SUPPORT
######################################################################

def sqlCreateStr(outfile=sys.stdout, fields=None, extraFields=None):
	'''
	Return the SQL CREATE command for this message type
	@param outfile: file like object to print to.
	@param fields: which fields to put in the create.  Defaults to all.
	@param extraFields: A sequence of tuples containing (name,sql type) for additional fields
	@return: sql create string
	@rtype: str

	@see: sqlCreate
	'''
	outfile.write(str(sqlCreate(fields,extraFields)))

def sqlCreate(fields=None, extraFields=None):
	'''
	Return the sqlhelp object to create the table.

	@param fields: which fields to put in the create.  Defaults to all.
	@param extraFields: A sequence of tuples containing (name,sql type) for additional fields
	@return: An object that can be used to generate a return
	@rtype: sqlhelp.create
	'''
	if None == fields: fields = fieldList
	import sqlhelp
	c = sqlhelp.create('sls_lockorder')
	if 'vessel' in fields: c.addVarChar(vessel,15)
	if 'direction' in fields: c.addBool('direction')
	if 'ETA_month' in fields: c.addInt ('ETA_month')
	if 'ETA_day' in fields: c.addInt ('ETA_day')
	if 'ETA_hour' in fields: c.addInt ('ETA_hour')
	if 'ETA_min' in fields: c.addInt ('ETA_min')
	if 'reserved' in fields: c.addInt ('reserved')

	return c

def sqlInsertStr(params, outfile=sys.stdout, extraParams=None):
	'''
	Return the SQL CREATE command for this message type
	@param params: dictionary of values keyed by field name
	@param outfile: file like object to print to.
	@param extraParams: A sequence of tuples containing (name,sql type) for additional fields
	@return: sql create string
	@rtype: str

	@see: sqlCreate
	'''
	outfile.write(str(sqlInsert(params,extraParams)))


def sqlInsert(params,extraParams=None):
	'''
	Give the SQL insert statement
	@param params: dict keyed by field name of values
	@param extraParams: any extra fields that you have created beyond the normal ais message fields
	@rtype: sqlhelp.insert
	@return: insert class instance
	@todo: allow optional type checking of params?
	@warning: this will take invalid keys happily and do what???
	'''
	import sqlhelp
	i = sqlhelp.insert('sls_lockorder')
	for key in params: i.add(key,params[key])
	if None != extraParams:
		for key in extraParams: 
			i.add(key,extraParams[key])

	return i


######################################################################
# UNIT TESTING
######################################################################
import unittest
def testParams():
	'''Return a params file base on the testvalue tags.
	@rtype: dict
	@return: params based on testvalue tags
	'''
	params = {}
	params['vessel'] = 'ICEBERG@@@@@@@@'
	params['direction'] = True
	params['ETA_month'] = 2
	params['ETA_day'] = 28
	params['ETA_hour'] = 23
	params['ETA_min'] = 45
	params['reserved'] = 0

	return params

class Testsls_lockorder(unittest.TestCase):
	'''Use testvalue tag text from each type to build test case the sls_lockorder message'''
	def testEncodeDecode(self):

		params = testParams()
		bits   = encode(params)
		r      = decode(bits)

		# Check that each parameter came through ok.
		self.failUnlessEqual(r['vessel'],params['vessel'])
		self.failUnlessEqual(r['direction'],params['direction'])
		self.failUnlessEqual(r['ETA_month'],params['ETA_month'])
		self.failUnlessEqual(r['ETA_day'],params['ETA_day'])
		self.failUnlessEqual(r['ETA_hour'],params['ETA_hour'])
		self.failUnlessEqual(r['ETA_min'],params['ETA_min'])
		self.failUnlessEqual(r['reserved'],params['reserved'])

def addMsgOptions(parser):
	parser.add_option('-d','--decode',dest='doDecode',default=False,action='store_true',
		help='decode a "sls_lockorder" AIS message')
	parser.add_option('-e','--encode',dest='doEncode',default=False,action='store_true',
		help='encode a "sls_lockorder" AIS message')
	parser.add_option('--vessel-field', dest='vesselField',default='@@@@@@@@@@@@@@@',metavar='aisstr6',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--direction-field', dest='directionField',metavar='bool',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--ETA_month-field', dest='ETA_monthField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--ETA_day-field', dest='ETA_dayField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--ETA_hour-field', dest='ETA_hourField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--ETA_min-field', dest='ETA_minField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')

############################################################
if __name__=='__main__':

	from optparse import OptionParser
	parser = OptionParser(usage="%prog [options]",
		version="%prog "+__version__)

	parser.add_option('--doc-test',dest='doctest',default=False,action='store_true',
		help='run the documentation tests')
	parser.add_option('--unit-test',dest='unittest',default=False,action='store_true',
		help='run the unit tests')
	parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
		help='Make the test output verbose')

	# FIX: remove nmea from binary messages.  No way to build the whole packet?
	# FIX: or build the surrounding msg 8 for a broadcast?
	typeChoices = ('binary','nmeapayload','nmea') # FIX: what about a USCG type message?
	parser.add_option('-t','--type',choices=typeChoices,type='choice',dest='ioType'
		,default='nmeapayload'
		,help='What kind of string to expect ('+', '.join(typeChoices)+') [default: %default]')


	outputChoices = ('std','html','csv','sql' )
	parser.add_option('-T','--output-type',choices=outputChoices,type='choice',dest='outputType'
		,default='std'
		,help='What kind of string to output ('+', '.join(outputChoices)+') [default: %default]')

	parser.add_option('-o','--output',dest='outputFileName',default=None,
			  help='Name of the python file to write [default: stdout]')

	parser.add_option('-f','--fields',dest='fieldList',default=None, action='append',
			  choices=fieldList,
			  help='Which fields to include in the output.  Currently only for csv output [default: all]')

	parser.add_option('-p','--print-csv-field-list',dest='printCsvfieldList',default=False,action='store_true',
			  help='Print the field name for csv')

	parser.add_option('-c','--sql-create',dest='sqlCreate',default=False,action='store_true',
			  help='Print out an sql create command for the table.')

	addMsgOptions(parser)

	(options,args) = parser.parse_args()
	success=True

	if options.doctest:
		import os; print os.path.basename(sys.argv[0]), 'doctests ...',
		sys.argv= [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		import doctest
		numfail,numtests=doctest.testmod()
		if numfail==0: print 'ok'
		else: 
			print 'FAILED'
			success=False

	if not success: sys.exit('Something Failed')
	del success # Hide success from epydoc

	if options.unittest:
		sys.argv = [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		unittest.main()

	outfile = sys.stdout
	if None!=options.outputFileName:
		outfile = file(options.outputFileName,'w')


	if options.doEncode:
		# First make sure all non required options are specified
		if None==options.vesselField: parser.error("missing value for vesselField")
		if None==options.directionField: parser.error("missing value for directionField")
		if None==options.ETA_monthField: parser.error("missing value for ETA_monthField")
		if None==options.ETA_dayField: parser.error("missing value for ETA_dayField")
		if None==options.ETA_hourField: parser.error("missing value for ETA_hourField")
		if None==options.ETA_minField: parser.error("missing value for ETA_minField")
		msgDict={
			'vessel': options.vesselField,
			'direction': options.directionField,
			'ETA_month': options.ETA_monthField,
			'ETA_day': options.ETA_dayField,
			'ETA_hour': options.ETA_hourField,
			'ETA_min': options.ETA_minField,
			'reserved': '0',
		}

		bits = encode(msgDict)
		if 'binary'==options.ioType: print str(bits)
		elif 'nmeapayload'==options.ioType:
		        # FIX: figure out if this might be necessary at compile time
			print "bitLen",len(bits)
			bitLen=len(bits)
			if bitLen%6!=0:
			    bits = bits + BitVector(size=(6 - (bitLen%6)))  # Pad out to multiple of 6
			print "result:",binary.bitvectoais6(bits)[0]


		# FIX: Do not emit this option for the binary message payloads.  Does not make sense.
		elif 'nmea'==options.ioType: sys.exit("FIX: need to implement this capability")
		else: sys.exit('ERROR: unknown ioType.  Help!')


	if options.sqlCreate:
		sqlCreateStr(outfile,options.fieldList)

	if options.printCsvfieldList:
		# Make a csv separated list of fields that will be displayed for csv
		if None == options.fieldList: options.fieldList = fieldList
		import StringIO
		buf = StringIO.StringIO()
		for field in options.fieldList:
			buf.write(field+',')
		result = buf.getvalue()
		if result[-1] == ',': print result[:-1]
		else: print result

	if options.doDecode:
		for msg in args:
			bv = None
			if    'binary'    == options.ioType: bv = BitVector(bitstring=msg)
			elif 'nmeapayload'== options.ioType: bv = binary.ais6tobitvec(msg)
			elif 'nmea'       == options.ioType: bv = binary.ais6tobitvec(msg.split(',')[5])
			else: sys.exit('ERROR: unknown ioType.  Help!')

			printFields(decode(bv),out=outfile,format=options.outputType,fieldList=options.fieldList)
